import numpy as np
import pandas as pd

from vresutils import reatlas as vreatlas, grid as vgrid, shapes as vshapes, landuse as vlanduse
from generation import resource
from load.europe import timeseries_europe

#%matplotlib inline
import matplotlib.pyplot as plt
import vresutils.plot as vplot

cutout = vreatlas.Cutout(username='becker', cutoutname='Europe_2011_2014')

ocean_depth_cutoff = 50 # in m or None

windturbines = dict(onshore='Vestas_V112_3MW', offshore='NREL_ReferenceTurbine_5MW_offshore')
solarpanel = dict(panel='KANENA', orientation='latitude_optimal')

## Potentials

#\*potentials are meant to hold a value of $x \, MW/km^2$ for each grid cell determined from landuse data, it is later multiplied by the cell areas to determine the maximum grid cell production. For vlanduse.corine_for_cutout to work properly one needs the python package rasterio and the gdalwarp command. vlanduse.corine_for_cutout takes a list of corine grid_codes as input.

#wind is preferred from PhD thesis by ed sharp p. 177
#
#solar comes from everything but forests and waters
#
#A homogeneous value of $1 \, MW/km^2$ is assumed for wind onshore/offshore and solar. FIX.

#%%time
onshorepotentials = vlanduse.corine_for_cutout(cutout, [12, 13, 14, 18, 23, 24, 25, 26, 27, 28, 29, 35, 36]) * 1.
offshorepotentials = 1.
solarpotentials = vlanduse.corine_for_cutout(cutout, [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18,
                                                      19, 20, 21, 22, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36]) * 1.

fig, axes = plt.subplots(1, 2, figsize=(12,6))
ax_iter = axes.flat
for ax, data in zip(axes.flat, [windpotentials, solarpotentials]):
    im = ax.imshow(data, extent=cutout.extent, cmap='viridis')
    vplot.draw_basemap(ax=ax)
fig.tight_layout()

## Time-Series

### Generation

partition = vreatlas.partition_from_emil(cutout)

#### Masks

#Masks are used to adjust the plain capacity factor layout (i.e. one turbine/m^2-panel at the location of a grid cell) by their geographical potential in terms of MW. They are constructed by combining landuse-specific $MW/km^2$ coefficients (\*potentials) with the actual grid cell area in $km^2$.

reatlas_cell_areas = np.asarray(map(vshapes.area, cutout.grid_cells())).reshape(cutout.shape)*1e-6

onshoremap = cutout.meta['onshoremap']
onshoremask = reatlas_cell_areas * onshorepotentials * onshoremap
solarmask = reatlas_cell_areas * solarpotentials * onshoremap
offshoremask = reatlas_cell_areas * offshorepotentials * (1. - onshoremap)

if ocean_depth_cutoff is not None:
    offshoremask *= (cutout.meta['heights'] > -ocean_depth_cutoff)

fig, axes = plt.subplots(3, 1, figsize=(8,12))
ax_iter = axes.flat
for ax, data in zip(axes.flat, [onshoremask, offshoremask, solarmask]):
    ax.imshow(data, extent=cutout.extent, cmap='viridis')
    vplot.draw_basemap(ax=ax)
fig.tight_layout()

#### REAtlas

#%%time
windlayout = resource.capacity_factor_layout(windturbines, cutout)
onshore = resource.unit_capacity_timeseries(dict(onshore=windturbines['onshore']), partition,
                                            onshoremask * windlayout, cutout)
offshore = resource.unit_capacity_timeseries(dict(offshore=windturbines['offshore']), partition,
                                             offshoremask * windlayout, cutout)

#%%time
solarlayout = resource.capacity_factor_layout(solarpanel, cutout)
solar = resource.unit_capacity_timeseries(solarpanel, partition, solarlayout * solarmask, cutout)

#### Determine peak production

def renorm(x): return x / x.max()
onshoremap = cutout.meta['onshoremap']
onshore_p_max_pu = partition.map(lambda x: (renorm(x * windlayout * onshoremap) * onshoremask).sum())
offshore_p_max_pu = partition.map(lambda x: (renorm(x * windlayout * (1.-onshoremap)) * offshoremask).sum())
solar_p_max_pu = partition.map(lambda x: (renorm(x * solarlayout * onshoremap) * solarmask).sum())

onshore_p_max_pu / 1e3

##### w/o renorm

pd.concat({'w/ renorm': onshore_p_max_pu,
           'w/o renorm': partition.map(lambda x: (x * onshoremask).sum())}, axis=1) /1e3

offshore_p_max_pu

solar_p_max_pu

### Load

load = timeseries_europe()

load

### Choose common data boundaries

graph = vgrid.eu()

timeseries = pd.Panel(dict(load=load, windon=onshore, windoff=offshore, solar=solar))
timeseries = timeseries.ix[:,:,graph.nodes()]
timeseries.dropna(axis=1, inplace=True)

timeseries

